Package: shiny
Type: Package
Title: Web Application Framework for R
Version: 1.0.3
Authors@R: c(
    person("Winston", "Chang", role = c("aut", "cre"), email = "winston@rstudio.com"),
    person("Joe", "Cheng", role = "aut", email = "joe@rstudio.com"),
    person("JJ", "Allaire", role = "aut", email = "jj@rstudio.com"),
    person("Yihui", "Xie", role = "aut", email = "yihui@rstudio.com"),
    person("Jonathan", "McPherson", role = "aut", email = "jonathan@rstudio.com"),
    person(family = "RStudio", role = "cph"),
    person(family = "jQuery Foundation", role = "cph",
    comment = "jQuery library and jQuery UI library"),
    person(family = "jQuery contributors", role = c("ctb", "cph"),
    comment = "jQuery library; authors listed in inst/www/shared/jquery-AUTHORS.txt"),
    person(family = "jQuery UI contributors", role = c("ctb", "cph"),
    comment = "jQuery UI library; authors listed in inst/www/shared/jqueryui/AUTHORS.txt"),
    person("Mark", "Otto", role = "ctb",
    comment = "Bootstrap library"),
    person("Jacob", "Thornton", role = "ctb",
    comment = "Bootstrap library"),
    person(family = "Bootstrap contributors", role = "ctb",
    comment = "Bootstrap library"),
    person(family = "Twitter, Inc", role = "cph",
    comment = "Bootstrap library"),
    person("Alexander", "Farkas", role = c("ctb", "cph"),
    comment = "html5shiv library"),
    person("Scott", "Jehl", role = c("ctb", "cph"),
    comment = "Respond.js library"),
    person("Stefan", "Petre", role = c("ctb", "cph"),
    comment = "Bootstrap-datepicker library"),
    person("Andrew", "Rowls", role = c("ctb", "cph"),
    comment = "Bootstrap-datepicker library"),
    person("Dave", "Gandy", role = c("ctb", "cph"),
    comment = "Font-Awesome font"),
    person("Brian", "Reavis", role = c("ctb", "cph"),
    comment = "selectize.js library"),
    person("Kristopher Michael", "Kowal", role = c("ctb", "cph"),
    comment = "es5-shim library"),
    person(family = "es5-shim contributors", role = c("ctb", "cph"),
    comment = "es5-shim library"),
    person("Denis", "Ineshin", role = c("ctb", "cph"),
    comment = "ion.rangeSlider library"),
    person("Sami", "Samhuri", role = c("ctb", "cph"),
    comment = "Javascript strftime library"),
    person(family = "SpryMedia Limited", role = c("ctb", "cph"),
    comment = "DataTables library"),
    person("John", "Fraser", role = c("ctb", "cph"),
    comment = "showdown.js library"),
    person("John", "Gruber", role = c("ctb", "cph"),
    comment = "showdown.js library"),
    person("Ivan", "Sagalaev", role = c("ctb", "cph"),
    comment = "highlight.js library"),
    person(family = "R Core Team", role = c("ctb", "cph"),
    comment = "tar implementation from R")
    )
Description: Makes it incredibly easy to build interactive web
    applications with R. Automatic "reactive" binding between inputs and
    outputs and extensive prebuilt widgets make it possible to build
    beautiful, responsive, and powerful applications with minimal effort.
License: GPL-3 | file LICENSE
Depends: R (>= 3.0.0), methods
Imports: utils, httpuv (>= 1.3.3), mime (>= 0.3), jsonlite (>= 0.9.16),
        xtable, digest, htmltools (>= 0.3.5), R6 (>= 2.0), sourcetools
Suggests: datasets, Cairo (>= 1.5-5), testthat, knitr (>= 1.6),
        markdown, rmarkdown, ggplot2, magrittr
URL: http://shiny.rstudio.com
BugReports: https://github.com/rstudio/shiny/issues
Collate: 'app.R' 'bookmark-state-local.R' 'stack.R' 'bookmark-state.R'
        'bootstrap-layout.R' 'conditions.R' 'map.R' 'globals.R'
        'utils.R' 'bootstrap.R' 'cache.R' 'diagnose.R' 'fileupload.R'
        'graph.R' 'reactives.R' 'reactive-domains.R' 'history.R'
        'hooks.R' 'html-deps.R' 'htmltools.R' 'image-interact-opts.R'
        'image-interact.R' 'imageutils.R' 'input-action.R'
        'input-checkbox.R' 'input-checkboxgroup.R' 'input-date.R'
        'input-daterange.R' 'input-file.R' 'input-numeric.R'
        'input-password.R' 'input-radiobuttons.R' 'input-select.R'
        'input-slider.R' 'input-submit.R' 'input-text.R'
        'input-textarea.R' 'input-utils.R' 'insert-ui.R' 'jqueryui.R'
        'middleware-shiny.R' 'middleware.R' 'modal.R' 'modules.R'
        'notifications.R' 'priorityqueue.R' 'progress.R' 'react.R'
        'render-plot.R' 'render-table.R' 'run-url.R' 'serializers.R'
        'server-input-handlers.R' 'server.R' 'shiny-options.R'
        'shiny.R' 'shinyui.R' 'shinywrappers.R' 'showcase.R' 'tar.R'
        'test-export.R' 'timer.R' 'update-input.R'
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2017-04-25 20:49:00 UTC; winston
Author: Winston Chang [aut, cre],
  Joe Cheng [aut],
  JJ Allaire [aut],
  Yihui Xie [aut],
  Jonathan McPherson [aut],
  RStudio [cph],
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/shared/jquery-AUTHORS.txt),
  jQuery UI contributors [ctb, cph] (jQuery UI library; authors listed in
    inst/www/shared/jqueryui/AUTHORS.txt),
  Mark Otto [ctb] (Bootstrap library),
  Jacob Thornton [ctb] (Bootstrap library),
  Bootstrap contributors [ctb] (Bootstrap library),
  Twitter, Inc [cph] (Bootstrap library),
  Alexander Farkas [ctb, cph] (html5shiv library),
  Scott Jehl [ctb, cph] (Respond.js library),
  Stefan Petre [ctb, cph] (Bootstrap-datepicker library),
  Andrew Rowls [ctb, cph] (Bootstrap-datepicker library),
  Dave Gandy [ctb, cph] (Font-Awesome font),
  Brian Reavis [ctb, cph] (selectize.js library),
  Kristopher Michael Kowal [ctb, cph] (es5-shim library),
  es5-shim contributors [ctb, cph] (es5-shim library),
  Denis Ineshin [ctb, cph] (ion.rangeSlider library),
  Sami Samhuri [ctb, cph] (Javascript strftime library),
  SpryMedia Limited [ctb, cph] (DataTables library),
  John Fraser [ctb, cph] (showdown.js library),
  John Gruber [ctb, cph] (showdown.js library),
  Ivan Sagalaev [ctb, cph] (highlight.js library),
  R Core Team [ctb, cph] (tar implementation from R)
Maintainer: Winston Chang <winston@rstudio.com>
Repository: CRAN
Date/Publication: 2017-04-26 17:13:08 UTC
Built: R 3.4.1; ; 2017-08-16 13:30:17 UTC; windows
