Package: shiny
Type: Package
Title: Web Application Framework for R
Version: 1.10.0
Authors@R: c(
    person("Winston", "Chang", role = c("aut", "cre"), email = "winston@posit.co", comment = c(ORCID = "0000-0002-1576-2126")),
    person("Joe", "Cheng", role = "aut", email = "joe@posit.co"),
    person("JJ", "Allaire", role = "aut", email = "jj@posit.co"),
    person("Carson", "Sievert", role = "aut", email = "carson@posit.co", comment = c(ORCID = "0000-0002-4958-2844")),
    person("Barret", "Schloerke", role = "aut", email = "barret@posit.co", comment = c(ORCID = "0000-0001-9986-114X")),
    person("Yihui", "Xie", role = "aut", email = "yihui@posit.co"),
    person("Jeff", "Allen", role = "aut"),
    person("Jonathan", "McPherson", role = "aut", email = "jonathan@posit.co"),
    person("Alan", "Dipert", role = "aut"),
    person("Barbara", "Borges", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person(family = "jQuery Foundation", role = "cph",
    comment = "jQuery library and jQuery UI library"),
    person(family = "jQuery contributors", role = c("ctb", "cph"),
    comment = "jQuery library; authors listed in inst/www/shared/jquery-AUTHORS.txt"),
    person(family = "jQuery UI contributors", role = c("ctb", "cph"),
    comment = "jQuery UI library; authors listed in inst/www/shared/jqueryui/AUTHORS.txt"),
    person("Mark", "Otto", role = "ctb",
    comment = "Bootstrap library"),
    person("Jacob", "Thornton", role = "ctb",
    comment = "Bootstrap library"),
    person(family = "Bootstrap contributors", role = "ctb",
    comment = "Bootstrap library"),
    person(family = "Twitter, Inc", role = "cph",
    comment = "Bootstrap library"),
    person("Prem Nawaz", "Khan", role = "ctb",
    comment = "Bootstrap accessibility plugin"),
    person("Victor", "Tsaran", role = "ctb",
    comment = "Bootstrap accessibility plugin"),
    person("Dennis", "Lembree", role = "ctb",
    comment = "Bootstrap accessibility plugin"),
    person("Srinivasu", "Chakravarthula", role = "ctb",
    comment = "Bootstrap accessibility plugin"),
    person("Cathy", "O'Connor", role = "ctb",
    comment = "Bootstrap accessibility plugin"),
    person(family = "PayPal, Inc", role = "cph",
    comment = "Bootstrap accessibility plugin"),
    person("Stefan", "Petre", role = c("ctb", "cph"),
    comment = "Bootstrap-datepicker library"),
    person("Andrew", "Rowls", role = c("ctb", "cph"),
    comment = "Bootstrap-datepicker library"),
    person("Brian", "Reavis", role = c("ctb", "cph"),
    comment = "selectize.js library"),
    person("Salmen", "Bejaoui", role = c("ctb", "cph"),
    comment = "selectize-plugin-a11y library"),
    person("Denis", "Ineshin", role = c("ctb", "cph"),
    comment = "ion.rangeSlider library"),
    person("Sami", "Samhuri", role = c("ctb", "cph"),
    comment = "Javascript strftime library"),
    person(family = "SpryMedia Limited", role = c("ctb", "cph"),
    comment = "DataTables library"),
    person("John", "Fraser", role = c("ctb", "cph"),
    comment = "showdown.js library"),
    person("John", "Gruber", role = c("ctb", "cph"),
    comment = "showdown.js library"),
    person("Ivan", "Sagalaev", role = c("ctb", "cph"),
    comment = "highlight.js library"),
    person(family = "R Core Team", role = c("ctb", "cph"),
    comment = "tar implementation from R")
    )
Description: Makes it incredibly easy to build interactive web
    applications with R. Automatic "reactive" binding between inputs and
    outputs and extensive prebuilt widgets make it possible to build
    beautiful, responsive, and powerful applications with minimal effort.
License: GPL-3 | file LICENSE
Depends: R (>= 3.0.2), methods
Imports: utils, grDevices, httpuv (>= 1.5.2), mime (>= 0.3), jsonlite
        (>= 0.9.16), xtable, fontawesome (>= 0.4.0), htmltools (>=
        0.5.4), R6 (>= 2.0), sourcetools, later (>= 1.0.0), promises
        (>= 1.3.2), tools, crayon, rlang (>= 0.4.10), fastmap (>=
        1.1.1), withr, commonmark (>= 1.7), glue (>= 1.3.2), bslib (>=
        0.6.0), cachem (>= 1.1.0), lifecycle (>= 0.2.0)
Suggests: coro (>= 1.1.0), datasets, DT, Cairo (>= 1.5-5), testthat (>=
        3.0.0), knitr (>= 1.6), markdown, rmarkdown, ggplot2, reactlog
        (>= 1.0.0), magrittr, yaml, future, dygraphs, ragg, showtext,
        sass
URL: https://shiny.posit.co/, https://github.com/rstudio/shiny
BugReports: https://github.com/rstudio/shiny/issues
Collate: 'globals.R' 'app-state.R' 'app_template.R' 'bind-cache.R'
        'bind-event.R' 'bookmark-state-local.R' 'bookmark-state.R'
        'bootstrap-deprecated.R' 'bootstrap-layout.R' 'conditions.R'
        'map.R' 'utils.R' 'bootstrap.R' 'busy-indicators-spinners.R'
        'busy-indicators.R' 'cache-utils.R' 'deprecated.R' 'devmode.R'
        'diagnose.R' 'extended-task.R' 'fileupload.R' 'graph.R'
        'reactives.R' 'reactive-domains.R' 'history.R' 'hooks.R'
        'html-deps.R' 'image-interact-opts.R' 'image-interact.R'
        'imageutils.R' 'input-action.R' 'input-checkbox.R'
        'input-checkboxgroup.R' 'input-date.R' 'input-daterange.R'
        'input-file.R' 'input-numeric.R' 'input-password.R'
        'input-radiobuttons.R' 'input-select.R' 'input-slider.R'
        'input-submit.R' 'input-text.R' 'input-textarea.R'
        'input-utils.R' 'insert-tab.R' 'insert-ui.R' 'jqueryui.R'
        'knitr.R' 'middleware-shiny.R' 'middleware.R' 'timer.R'
        'shiny.R' 'mock-session.R' 'modal.R' 'modules.R'
        'notifications.R' 'priorityqueue.R' 'progress.R' 'react.R'
        'reexports.R' 'render-cached-plot.R' 'render-plot.R'
        'render-table.R' 'run-url.R' 'runapp.R' 'serializers.R'
        'server-input-handlers.R' 'server-resource-paths.R' 'server.R'
        'shiny-options.R' 'shiny-package.R' 'shinyapp.R' 'shinyui.R'
        'shinywrappers.R' 'showcase.R' 'snapshot.R' 'staticimports.R'
        'tar.R' 'test-export.R' 'test-server.R' 'test.R'
        'update-input.R' 'utils-lang.R' 'version_bs_date_picker.R'
        'version_ion_range_slider.R' 'version_jquery.R'
        'version_jqueryui.R' 'version_selectize.R' 'version_strftime.R'
        'viewer.R'
RoxygenNote: 7.3.2
Encoding: UTF-8
RdMacros: lifecycle
Config/testthat/edition: 3
Config/Needs/check: shinytest2
NeedsCompilation: no
Packaged: 2024-12-13 21:47:15 UTC; cpsievert
Author: Winston Chang [aut, cre] (<https://orcid.org/0000-0002-1576-2126>),
  Joe Cheng [aut],
  JJ Allaire [aut],
  Carson Sievert [aut] (<https://orcid.org/0000-0002-4958-2844>),
  Barret Schloerke [aut] (<https://orcid.org/0000-0001-9986-114X>),
  Yihui Xie [aut],
  Jeff Allen [aut],
  Jonathan McPherson [aut],
  Alan Dipert [aut],
  Barbara Borges [aut],
  Posit Software, PBC [cph, fnd],
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/shared/jquery-AUTHORS.txt),
  jQuery UI contributors [ctb, cph] (jQuery UI library; authors listed in
    inst/www/shared/jqueryui/AUTHORS.txt),
  Mark Otto [ctb] (Bootstrap library),
  Jacob Thornton [ctb] (Bootstrap library),
  Bootstrap contributors [ctb] (Bootstrap library),
  Twitter, Inc [cph] (Bootstrap library),
  Prem Nawaz Khan [ctb] (Bootstrap accessibility plugin),
  Victor Tsaran [ctb] (Bootstrap accessibility plugin),
  Dennis Lembree [ctb] (Bootstrap accessibility plugin),
  Srinivasu Chakravarthula [ctb] (Bootstrap accessibility plugin),
  Cathy O'Connor [ctb] (Bootstrap accessibility plugin),
  PayPal, Inc [cph] (Bootstrap accessibility plugin),
  Stefan Petre [ctb, cph] (Bootstrap-datepicker library),
  Andrew Rowls [ctb, cph] (Bootstrap-datepicker library),
  Brian Reavis [ctb, cph] (selectize.js library),
  Salmen Bejaoui [ctb, cph] (selectize-plugin-a11y library),
  Denis Ineshin [ctb, cph] (ion.rangeSlider library),
  Sami Samhuri [ctb, cph] (Javascript strftime library),
  SpryMedia Limited [ctb, cph] (DataTables library),
  John Fraser [ctb, cph] (showdown.js library),
  John Gruber [ctb, cph] (showdown.js library),
  Ivan Sagalaev [ctb, cph] (highlight.js library),
  R Core Team [ctb, cph] (tar implementation from R)
Maintainer: Winston Chang <winston@posit.co>
Repository: CRAN
Date/Publication: 2024-12-14 00:10:02 UTC
Built: R 4.4.2; ; 2024-12-17 11:18:44 UTC; unix
