This a full jQuery UI build, downloaded from:
https://jqueryui.com/resources/download/jquery-ui-1.12.1.zip

The copy of jQuery that is bundled with the download, under external/, is not
included because Shiny already has its own copy of jQuery.
